"""CLI commands package.

This package contains all CLI command implementations organized by functionality:
- parse_cmd.py: Parse YARA files into various output formats
- format_cmd.py: Basic YARA file formatting
- fmt_cmd.py: Advanced AST-based formatting (like black)
- diff_cmd.py: AST-based diff showing logical vs style changes
- bench_cmd.py: Performance benchmarking
- libyara_cmd.py: LibYARA integration (compile, scan, optimize)
- analyze.py: Best practices and optimization analysis
- validate.py: Cross-validation with libyara
- metrics.py: Complexity and maintainability metrics
- semantic.py: Semantic analysis
- serialize.py: AST serialization
- roundtrip.py: Round-trip testing
- workspace.py: Workspace management
- fluent.py: Fluent API demonstration
- yaral.py: YARA-L dialect support
- yarax.py: YARA-X dialect support
- lsp.py: Language Server Protocol
- optimize.py: Rule optimization
- performance.py: Performance profiling
- performance_check.py: Performance checking
"""
